"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const lodash = require("lodash");
const cli_framework_1 = require("@ionic/cli-framework");
const format_1 = require("@ionic/cli-framework/utils/format");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
class ConfigSetCommand extends command_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'set',
                type: 'global',
                summary: 'Set config values',
                description: `
By default, this command sets JSON properties in your project's ${chalk_1.default.bold(cli_utils_1.PROJECT_FILE)} file.

For ${chalk_1.default.green('--global')} config, the CLI sets properties in the global CLI config file (${chalk_1.default.bold('~/.ionic/config.json')}).

For nested properties, separate nest levels with dots. For example, the property name ${chalk_1.default.green('user.email')} will look in the ${chalk_1.default.bold('user')} object (a root-level field in the global CLI config file) for the ${chalk_1.default.bold('email')} field.

${chalk_1.default.green('ionic config set')} will attempt to coerce ${chalk_1.default.green('value')} into a suitable JSON type. If it is JSON-parsable, such as ${chalk_1.default.green('true')} or ${chalk_1.default.green('[]')}, it takes the parsed result. Otherwise, the value is interpreted as a string. For stricter input, use ${chalk_1.default.green('--json')}, which will error with non-JSON values.

By default, if ${chalk_1.default.green('property')} exists and is an object or an array, the value is not overwritten. To disable this check and always overwrite the property, use ${chalk_1.default.green('--force')}.
      `,
                inputs: [
                    {
                        name: 'property',
                        summary: 'The property name you wish to set',
                        validators: [cli_framework_1.validators.required],
                    },
                    {
                        name: 'value',
                        summary: 'The new value of the given property',
                        validators: [cli_framework_1.validators.required],
                    },
                ],
                options: [
                    {
                        name: 'global',
                        summary: 'Use global CLI config',
                        type: Boolean,
                        aliases: ['g'],
                    },
                    {
                        name: 'json',
                        summary: `Always interpret ${chalk_1.default.green('value')} as JSON`,
                        type: Boolean,
                    },
                    {
                        name: 'force',
                        summary: 'Always overwrite existing values',
                        type: Boolean,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                ],
                exampleCommands: ['name newAppName', 'name "\\"newAppName\\"" --json', '-g interactive true'],
            };
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [p] = inputs;
            let [, v] = inputs;
            const { global, json, force } = options;
            if (!global && !this.env.project.directory) {
                throw new errors_1.FatalException(`Sorry--this won't work outside an Ionic project directory. Did you mean to set global config using ${chalk_1.default.green('--global')}?`);
            }
            const file = global ? this.env.config : this.env.project;
            const config = yield file.load();
            const oldValue = lodash.get(config, p);
            if (!v.match(/^\d+e\d+$/)) {
                try {
                    v = JSON.parse(v);
                }
                catch (e) {
                    if (!(e instanceof SyntaxError)) {
                        throw e;
                    }
                    if (json) {
                        throw new errors_1.FatalException(`${chalk_1.default.green('--json')}: ${chalk_1.default.green(v)} is invalid JSON: ${chalk_1.default.red(String(e))}`);
                    }
                }
            }
            const newValue = v;
            if (oldValue && typeof oldValue === 'object' && !force) {
                throw new errors_1.FatalException(`Sorry--will not override objects or arrays without ${chalk_1.default.green('--force')}.\n` +
                    `Value of ${chalk_1.default.green(p)} is: ${chalk_1.default.bold(JSON.stringify(oldValue))}`);
            }
            const valueChanged = oldValue !== newValue;
            lodash.set(config, p, newValue);
            yield file.save();
            if (valueChanged) {
                this.env.log.ok(`${chalk_1.default.green(p)} set to ${chalk_1.default.green(JSON.stringify(v))} in ${chalk_1.default.bold(format_1.prettyPath(file.filePath))}!`);
            }
            else {
                this.env.log.msg(`${chalk_1.default.green(p)} is already set to ${chalk_1.default.bold(JSON.stringify(v))}.`);
            }
        });
    }
}
exports.ConfigSetCommand = ConfigSetCommand;
