"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const array_1 = require("@ionic/cli-framework/utils/array");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const format_1 = require("@ionic/cli-framework/utils/format");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const executor_1 = require("@ionic/cli-utils/lib/executor");
exports.COMMON_CORDOVA_BUILD_COMMAND_OPTIONS = [
    {
        name: 'debug',
        summary: 'Create a debug build',
        type: Boolean,
        groups: [cli_utils_1.OptionGroup.Cordova],
        hint: 'cordova',
    },
    {
        name: 'release',
        summary: 'Create a release build',
        type: Boolean,
        groups: [cli_utils_1.OptionGroup.Cordova],
        hint: 'cordova',
    },
    {
        name: 'device',
        summary: 'Create a build for a device',
        type: Boolean,
        groups: [cli_utils_1.OptionGroup.Cordova],
        hint: 'cordova',
    },
    {
        name: 'emulator',
        summary: 'Create a build for an emulator',
        type: Boolean,
        groups: [cli_utils_1.OptionGroup.Cordova],
        hint: 'cordova',
    },
    {
        name: 'buildConfig',
        summary: 'Use the specified build configuration',
        groups: [cli_utils_1.OptionGroup.Advanced, cli_utils_1.OptionGroup.Cordova],
        hint: 'cordova',
    },
];
exports.CORDOVA_BUILD_EXAMPLE_COMMANDS = [
    'ios',
    'ios --prod --release',
    'ios --prod --release -- --developmentTeam="ABCD" --codeSignIdentity="iPhone Developer" --packageType="app-store"',
    'ios --buildConfig=build.json',
    'ios --prod --release --buildConfig=build.json',
    'android',
    'android --prod --release -- -- --keystore=filename.keystore --alias=myalias',
    'android --prod --release -- -- --minSdkVersion=21',
    'android --prod --release -- -- --versionCode=55',
    'android --prod --release -- -- --gradleArg=-PcdvBuildMultipleApks=true',
    'android --buildConfig=build.json',
    'android --prod --release --buildConfig=build.json',
];
class CordovaCommand extends command_1.Command {
    checkCordova(runinfo) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ADD_CORDOVA_ENGINE_HOOK, REMOVE_CORDOVA_ENGINE_HOOK, HOOKS_PKG, locateHook } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/hooks'));
            const project = yield this.env.project.load();
            if (project.integrations.cordova && project.integrations.cordova.enabled === false) {
                return;
            }
            // TODO: better way?
            let hooksNeedInstalling = false;
            if (this.env.project.type === 'angular') {
                const allowedByConfig = !project.integrations.cordova || project.integrations.cordova.setupEngineHooks !== false;
                const hooksFound = (locateHook(this.env.project.directory, array_1.conform(project.hooks['build:before']), ADD_CORDOVA_ENGINE_HOOK) >= 0 &&
                    locateHook(this.env.project.directory, array_1.conform(project.hooks['build:after']), REMOVE_CORDOVA_ENGINE_HOOK) >= 0 &&
                    locateHook(this.env.project.directory, array_1.conform(project.hooks['serve:before']), ADD_CORDOVA_ENGINE_HOOK) >= 0 &&
                    locateHook(this.env.project.directory, array_1.conform(project.hooks['serve:after']), REMOVE_CORDOVA_ENGINE_HOOK) >= 0);
                hooksNeedInstalling = allowedByConfig && !hooksFound;
                if (hooksNeedInstalling) {
                    this.env.log.info(`Cordova engine hooks not found in existing Cordova integration. Re-enabling integration.\n` +
                        `This process will make sure the ${chalk_1.default.bold(HOOKS_PKG)} package is installed and that the hooks are defined in ${chalk_1.default.bold(cli_utils_1.PROJECT_FILE)}. To disable this process, run: ${chalk_1.default.green('ionic config set integrations.cordova.setupEngineHooks false')}`);
                }
            }
            if (!project.integrations.cordova || hooksNeedInstalling) {
                yield executor_1.runCommand(runinfo, ['integrations', 'enable', 'cordova']);
            }
        });
    }
    preRunChecks(runinfo) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ConfigXml } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/integrations/cordova/config'));
            yield this.checkCordova(runinfo);
            // Check for www folder
            if (this.env.project.directory) {
                const wwwPath = path.join(this.env.project.directory, 'www');
                const wwwExists = yield fs_1.pathExists(wwwPath); // TODO: hard-coded
                if (!wwwExists) {
                    this.env.tasks.next(`Creating ${chalk_1.default.bold(format_1.prettyPath(wwwPath))} directory for you`);
                    yield fs_1.fsMkdir(wwwPath, 0o777);
                    this.env.tasks.end();
                }
            }
            const conf = yield ConfigXml.load(this.env.project.directory);
            conf.resetContentSrc();
            yield conf.save();
        });
    }
    runCordova(argList, _a = {}) {
        var { fatalOnNotFound = false, truncateErrorOutput = 5000 } = _a, options = tslib_1.__rest(_a, ["fatalOnNotFound", "truncateErrorOutput"]);
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ERROR_SHELL_COMMAND_NOT_FOUND } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/shell'));
            const { pkgManagerArgs } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/npm'));
            const config = yield this.env.config.load();
            const { npmClient } = config;
            try {
                yield this.env.shell.run('cordova', argList, Object.assign({ fatalOnNotFound, truncateErrorOutput }, options));
            }
            catch (e) {
                if (e === ERROR_SHELL_COMMAND_NOT_FOUND) {
                    const cdvInstallArgs = yield pkgManagerArgs(npmClient, { command: 'install', pkg: 'cordova', global: true });
                    throw new errors_1.FatalException(`The Cordova CLI was not found on your PATH. Please install Cordova globally:\n` +
                        `${chalk_1.default.green(cdvInstallArgs.join(' '))}\n`);
                }
                if (options.fatalOnError) {
                    this.env.log.nl();
                    this.env.log.error('Cordova encountered an error.\nYou may get more insight by running the Cordova command above directly.\n');
                }
                throw e;
            }
        });
    }
    checkForPlatformInstallation(runPlatform) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (runPlatform) {
                const { getPlatforms, installPlatform } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/integrations/cordova/project'));
                const platforms = yield getPlatforms(this.env.project.directory);
                if (!platforms.includes(runPlatform)) {
                    yield installPlatform(this.env, runPlatform);
                }
            }
        });
    }
}
exports.CordovaCommand = CordovaCommand;
