"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const cli_framework_1 = require("@ionic/cli-framework");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const executor_1 = require("@ionic/cli-utils/lib/executor");
const uuid_1 = require("@ionic/cli-utils/lib/utils/uuid");
class LoginCommand extends command_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'login',
                type: 'global',
                summary: 'Login to Ionic Pro',
                description: `
Authenticate with Ionic Pro and retrieve a user token, which is stored in the CLI config.

Alternatively, set the ${chalk_1.default.green('IONIC_TOKEN')} environment variable and the CLI will automatically authenticate you. (You can also use ${chalk_1.default.green('IONIC_EMAIL')} and ${chalk_1.default.green('IONIC_PASSWORD')}.)

If you need to create an Ionic Pro account, use ${chalk_1.default.green('ionic signup')}.
      `,
                exampleCommands: ['', 'john@example.com', 'hello@example.com secret'],
                inputs: [
                    {
                        name: 'email',
                        summary: 'Your email address',
                        validators: [cli_framework_1.validators.required, cli_framework_1.validators.email],
                        private: true,
                    },
                    {
                        name: 'password',
                        summary: 'Your password',
                        validators: [cli_framework_1.validators.required],
                        private: true,
                    },
                ],
                options: [
                    {
                        name: 'email',
                        summary: '',
                        private: true,
                        groups: [cli_utils_1.OptionGroup.Hidden],
                    },
                    {
                        name: 'password',
                        summary: '',
                        private: true,
                        groups: [cli_utils_1.OptionGroup.Hidden],
                    },
                ],
            };
        });
    }
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [email] = inputs;
            const config = yield this.env.config.load();
            if (yield this.env.session.isLoggedIn()) {
                const extra = !inputs[0] || !inputs[1] ? 'Prompting for new credentials.' : 'Attempting login.';
                this.env.log.warn(`You are already logged in${config.user.email ? ' as ' + chalk_1.default.bold(config.user.email) : ''}! ${this.env.flags.interactive ? extra : ''}`);
            }
            else {
                this.env.log.msg(`Log into your Ionic Pro account\n` +
                    `If you don't have one yet, create yours by running: ${chalk_1.default.green(`ionic signup`)}\n`);
            }
            if (options['email'] || options['password']) {
                const extra = this.env.flags.interactive ? 'You will be prompted to provide credentials. Alternatively, you can try this:' : 'Try this:';
                this.env.log.warn(`${chalk_1.default.green('email')} and ${chalk_1.default.green('password')} are command arguments, not options. ${extra}\n` +
                    `${chalk_1.default.green('ionic login ' + (options['email'] ? options['email'] : email) + ' *****')}\n`);
            }
            // TODO: combine with promptToLogin ?
            if (!inputs[0]) {
                const email = yield this.env.prompt({
                    type: 'input',
                    name: 'email',
                    message: 'Email:',
                    default: options['email'],
                    validate: v => cli_framework_1.validators.required(v) && cli_framework_1.validators.email(v),
                });
                inputs[0] = email;
            }
            if (!inputs[1]) {
                const password = yield this.env.prompt({
                    type: 'password',
                    name: 'password',
                    message: 'Password:',
                    mask: '*',
                    validate: v => cli_framework_1.validators.required(v),
                });
                inputs[1] = password;
            }
        });
    }
    run(inputs, options, runinfo) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [email, password] = inputs;
            const config = yield this.env.config.load();
            if (yield this.env.session.isLoggedIn()) {
                this.env.log.msg('Logging you out.');
                yield executor_1.runCommand(runinfo, ['logout']);
                config.tokens.telemetry = uuid_1.generateUUID();
            }
            yield this.env.session.login(email, password);
            this.env.log.ok('You are logged in!');
        });
    }
}
exports.LoginCommand = LoginCommand;
