"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const Debug = require("debug");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const format_1 = require("@ionic/cli-framework/utils/format");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const debug = Debug('ionic:cli:commands:monitoring:syncmaps');
const SOURCEMAP_DIRECTORY = '.sourcemaps';
class MonitoringSyncSourcemapsCommand extends command_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'syncmaps',
                type: 'project',
                summary: 'Build & upload sourcemaps to Ionic Pro Monitoring service',
                description: `
By default, ${chalk_1.default.green('ionic monitoring syncmaps')} will upload the sourcemap files within ${chalk_1.default.bold(SOURCEMAP_DIRECTORY)}. To optionally perform a production build before uploading sourcemaps, specify the ${chalk_1.default.green('--build')} flag.
      `,
                inputs: [
                    {
                        name: 'snapshot_id',
                        summary: `Specify a Snapshot ID to associate the uploaded sourcemaps with`,
                    },
                ],
                options: [
                    {
                        name: 'build',
                        summary: 'Invoke a production Ionic build',
                        type: Boolean,
                    },
                ],
            };
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.env.session.getUserToken();
            const appId = yield this.env.project.loadAppId();
            const [snapshotId] = inputs;
            const doBuild = options.build ? true : false;
            const { ConfigXml } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/integrations/cordova/config'));
            const conf = yield ConfigXml.load(this.env.project.directory);
            const cordovaInfo = conf.getProjectInfo();
            const appVersion = cordovaInfo.version;
            const commitHash = (yield this.env.shell.output('git', ['rev-parse', 'HEAD'], { cwd: this.env.project.directory })).trim();
            debug(`Commit hash: ${chalk_1.default.bold(commitHash)}`);
            const sourcemapsDir = path.resolve(this.env.project.directory, SOURCEMAP_DIRECTORY);
            let sourcemapsExist = yield fs_1.pathExists(sourcemapsDir);
            if (doBuild || !sourcemapsExist) {
                const { build } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/build'));
                yield build(this.env, [], { _: [], prod: true });
            }
            sourcemapsExist = yield fs_1.pathExists(sourcemapsDir);
            if (sourcemapsExist) {
                this.env.log.msg(`Using existing sourcemaps in ${chalk_1.default.bold(format_1.prettyPath(sourcemapsDir))}`);
            }
            else {
                throw new errors_1.FatalException(`Cannot find directory: ${chalk_1.default.bold(format_1.prettyPath(sourcemapsDir))}.\n` +
                    `Make sure you have the latest ${chalk_1.default.bold('@ionic/app-scripts')}. Then, re-run this command.`);
            }
            let count = 0;
            this.env.tasks.next('Syncing sourcemaps');
            const sourcemapFiles = (yield fs_1.readDir(sourcemapsDir)).filter(f => f.endsWith('.js.map'));
            debug(`Found ${sourcemapFiles.length} sourcemap files: ${sourcemapFiles.map(f => chalk_1.default.bold(f)).join(', ')}`);
            yield Promise.all(sourcemapFiles.map((f) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.syncSourcemap(path.resolve(sourcemapsDir, f), snapshotId, appVersion, commitHash, appId, token);
                count += 1;
                this.env.tasks.updateMsg(`Syncing sourcemaps: ${chalk_1.default.bold(`${count} / ${sourcemapFiles.length}`)}`);
            })));
            this.env.tasks.updateMsg(`Syncing sourcemaps: ${chalk_1.default.bold(`${sourcemapFiles.length} / ${sourcemapFiles.length}`)}`);
            this.env.tasks.end();
            const details = format_1.columnar([
                ['App ID', chalk_1.default.bold(appId)],
                ['App Version', chalk_1.default.bold(appVersion)],
                ['Bundle ID', chalk_1.default.bold(cordovaInfo.id)],
                ['Snapshot ID', snapshotId ? chalk_1.default.bold(snapshotId) : chalk_1.default.dim('not set')],
            ], { vsep: ':' });
            this.env.log.ok(`Sourcemaps synced!\n` +
                details + '\n\n' +
                `See the Error Monitoring docs for usage information and next steps: ${chalk_1.default.bold('https://ionicframework.com/docs/pro/monitoring')}`);
        });
    }
    syncSourcemap(file, snapshotId, appVersion, commitHash, appId, token) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { req } = yield this.env.client.make('POST', `/monitoring/${appId}/sourcemaps`);
            req
                .set('Authorization', `Bearer ${token}`)
                .send({
                name: path.basename(file),
                version: appVersion,
                commit: commitHash,
                snapshot_id: snapshotId,
            });
            try {
                const res = yield this.env.client.do(req);
                return this.uploadSourcemap(res, file);
            }
            catch (e) {
                if (cli_utils_1.isSuperAgentError(e)) {
                    this.env.log.error(`Unable to sync map ${file}: ` + e.message);
                    if (e.response.status === 401) {
                        this.env.log.error('Try logging out and back in again.');
                    }
                    this.env.tasks.fail();
                }
                else {
                    throw e;
                }
            }
        });
    }
    uploadSourcemap(sourcemap, file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { createRequest } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/http'));
            const sm = sourcemap;
            const fileData = yield fs_1.fsReadFile(file, { encoding: 'utf8' });
            const sourcemapPost = sm.data.sourcemap_post;
            const c = yield this.env.config.load();
            const { req } = yield createRequest('POST', sourcemapPost.url, c);
            req
                .field(sourcemapPost.fields)
                .field('file', fileData);
            const res = yield req;
            if (res.status !== 204) {
                throw new errors_1.FatalException(`Unexpected status code from AWS: ${res.status}`);
            }
        });
    }
}
exports.MonitoringSyncSourcemapsCommand = MonitoringSyncSourcemapsCommand;
