"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const util = require("util");
const chalk_1 = require("chalk");
const Debug = require("debug");
const cli_framework_1 = require("@ionic/cli-framework");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const npm_1 = require("@ionic/cli-framework/utils/npm");
const cli_utils_1 = require("@ionic/cli-utils");
const executor_1 = require("@ionic/cli-utils/lib/executor");
const init_1 = require("@ionic/cli-utils/lib/init");
const commands_1 = require("./commands");
const debug = Debug('ionic:cli');
const PACKAGE_ROOT_PATH = path.dirname(path.dirname(__filename));
const PACKAGE_JSON_PATH = path.resolve(PACKAGE_ROOT_PATH, 'package.json');
let _pkg;
let _executor;
function loadPackageJson() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!_pkg) {
            _pkg = yield npm_1.readPackageJsonFile(PACKAGE_JSON_PATH);
        }
        return _pkg;
    });
}
function generateContext() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const pkg = yield loadPackageJson();
        if (!pkg.bin || !pkg.bin.ionic) {
            throw new Error(`Missing "${chalk_1.default.bold('bin.ionic')}" in Ionic CLI package.json`);
        }
        if (!pkg.main) {
            throw new Error(`Missing "${chalk_1.default.bold('main')}" in Ionic CLI package.json`);
        }
        return {
            binPath: path.resolve(PACKAGE_ROOT_PATH, pkg.bin.ionic),
            libPath: path.resolve(PACKAGE_ROOT_PATH, pkg.main),
            execPath: process.cwd(),
            version: pkg.version,
        };
    });
}
exports.generateContext = generateContext;
function loadExecutor(ctx, pargv, env) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!_executor) {
            const namespace = new commands_1.IonicNamespace(undefined, yield cli_utils_1.generateIonicEnvironment(ctx, pargv, env));
            _executor = new executor_1.Executor({ namespace });
        }
        return _executor;
    });
}
exports.loadExecutor = loadExecutor;
function run(pargv, env) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const now = new Date();
        let err;
        let executor;
        pargv = init_1.modifyArguments(pargv);
        try {
            executor = yield loadExecutor(yield generateContext(), pargv, env);
        }
        catch (e) {
            process.stderr.write(`${e.message ? e.message : (e.stack ? e.stack : e)}\n`);
            process.exitCode = 1;
            return;
        }
        const ienv = executor.namespace.env;
        if (pargv[0] !== '_') {
            try {
                const config = yield ienv.config.load();
                debug(util.inspect(ienv.ctx, { breakLength: Infinity, colors: chalk_1.default.enabled }));
                if (env['IONIC_TOKEN']) {
                    const wasLoggedIn = yield ienv.session.isLoggedIn();
                    debug(`${chalk_1.default.bold('IONIC_TOKEN')} environment variable detected`);
                    if (config.tokens.user !== env['IONIC_TOKEN']) {
                        debug(`${chalk_1.default.bold('IONIC_TOKEN')} mismatch with current session--attempting login`);
                        yield ienv.session.tokenLogin(env['IONIC_TOKEN']);
                        if (wasLoggedIn) {
                            ienv.log.info(`You have been logged out--using ${chalk_1.default.bold('IONIC_TOKEN')} environment variable`);
                        }
                    }
                }
                else if (env['IONIC_EMAIL'] && env['IONIC_PASSWORD']) {
                    debug(`${chalk_1.default.bold('IONIC_EMAIL')} / ${chalk_1.default.bold('IONIC_PASSWORD')} environment variables detected`);
                    if (config.user.email !== env['IONIC_EMAIL']) {
                        debug(`${chalk_1.default.bold('IONIC_EMAIL')} mismatch with current session--attempting login`);
                        try {
                            yield ienv.session.login(env['IONIC_EMAIL'], env['IONIC_PASSWORD']);
                        }
                        catch (e) {
                            ienv.log.error(`Error occurred during automatic login via ${chalk_1.default.bold('IONIC_EMAIL')} / ${chalk_1.default.bold('IONIC_PASSWORD')} environment variables.`);
                            throw e;
                        }
                    }
                }
                if (ienv.project.directory) {
                    const nodeModulesExists = yield fs_1.pathExists(path.join(ienv.project.directory, 'node_modules'));
                    if (!nodeModulesExists) {
                        const confirm = yield ienv.prompt({
                            type: 'confirm',
                            name: 'confirm',
                            message: `Looks like a fresh checkout! No ${chalk_1.default.green('./node_modules')} directory found. Would you like to install project dependencies?`,
                        });
                        if (confirm) {
                            ienv.log.msg('Installing dependencies may take several minutes!');
                            const { pkgManagerArgs } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/npm'));
                            const { npmClient } = config;
                            const [installer, ...installerArgs] = yield pkgManagerArgs(npmClient, { command: 'install' });
                            yield ienv.shell.run(installer, installerArgs, {});
                        }
                    }
                }
                const parsedArgs = cli_framework_1.stripOptions(pargv, { includeSeparated: false });
                // If an legacy command is being executed inform the user that there is a new command available
                const foundCommand = init_1.mapLegacyCommand(parsedArgs[0]);
                if (foundCommand) {
                    ienv.log.msg(`The ${chalk_1.default.green(parsedArgs[0])} command has been renamed. To find out more, run:\n\n` +
                        `    ${chalk_1.default.green(`ionic ${foundCommand} --help`)}\n\n`);
                }
                else {
                    yield executor.execute(pargv, env);
                    config.state.lastCommand = now.toISOString();
                }
                if (ienv.flags.interactive) {
                    const updateNotifier = yield Promise.resolve().then(() => require('update-notifier'));
                    updateNotifier({ pkg: yield loadPackageJson() }).notify({ isGlobal: true });
                }
            }
            catch (e) {
                err = e;
            }
        }
        try {
            yield Promise.all([ienv.config.save(), ienv.project.save()]);
        }
        catch (e) {
            ienv.log.error(String(e.stack ? e.stack : e));
        }
        if (err) {
            ienv.tasks.fail();
            process.exitCode = 1;
            if (err instanceof cli_framework_1.InputValidationError) {
                for (const e of err.errors) {
                    ienv.log.error(e.message);
                }
                ienv.log.msg(`Use the ${chalk_1.default.green('--help')} flag for more details.`);
            }
            else if (cli_utils_1.isSuperAgentError(err)) {
                const { formatSuperAgentError } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/http'));
                ienv.log.rawmsg(formatSuperAgentError(err));
            }
            else if (err.code && err.code === 'ENOTFOUND' || err.code === 'ECONNREFUSED') {
                ienv.log.error(`Network connectivity error occurred, are you offline?\n` +
                    `If you are behind a firewall and need to configure proxy settings, see: ${chalk_1.default.bold('https://ionicframework.com/docs/cli/configuring.html#using-a-proxy')}\n\n` +
                    chalk_1.default.red(String(err.stack ? err.stack : err)));
            }
            else if (cli_utils_1.isExitCodeException(err)) {
                process.exitCode = err.exitCode;
                if (err.message) {
                    if (err.exitCode > 0) {
                        ienv.log.error(err.message);
                    }
                    else {
                        ienv.log.msg(err.message);
                    }
                }
            }
            else if (err instanceof cli_framework_1.BaseError) {
                ienv.log.error(err.message);
            }
            else {
                ienv.log.msg(chalk_1.default.red(String(err.stack ? err.stack : err)));
                if (err.stack) {
                    debug(chalk_1.default.red(String(err.stack)));
                }
            }
        }
        ienv.close();
    });
}
exports.run = run;
function receive(msg) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!_executor) {
            throw new Error('Executor not initialized.');
        }
        const env = _executor.namespace.env;
        if (msg.type === 'telemetry') {
            const { sendCommand } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/telemetry'));
            yield sendCommand({
                getInfo: env.getInfo,
                client: env.client,
                config: env.config,
                ctx: env.ctx,
                project: env.project,
                session: env.session,
            }, msg.data.command, msg.data.args);
        }
    });
}
exports.receive = receive;
